﻿using System;
using System.ServiceModel;
using System.Threading;
using System.Windows.Forms;
using Artech.ConcurrentServiceInvocation.Service.Interface;
namespace Artech.ConcurrentServiceInvocation.Client
{
    public partial class MonitorForm : Form
    {
        private SynchronizationContext _syncContext;
        private ChannelFactory<ICalculator> _channelFactory;
        private static int clientIdIndex = 0;

        public MonitorForm()
        {
            InitializeComponent();
        }

        private void MonitorForm_Load(object sender, EventArgs e)
        {
            string header = string.Format("{0, -13}{1, -22}{2}", "Client", "Time", "Event");
            this.listBoxExecutionProgress.Items.Add(header);
            _syncContext = SynchronizationContext.Current;
            _channelFactory = new ChannelFactory<ICalculator>("calculatorservice");

            EventMonitor.MonitoringNotificationSended += ReceiveMonitoringNotification;
            this.Disposed += delegate
            {
                EventMonitor.MonitoringNotificationSended -= ReceiveMonitoringNotification;
                _channelFactory.Close();
            };

            ConcurrentCallViaDiffProxy();
        }

        private void ConcurrentCallViaSameProxy()
        {
            ICalculator proxy = _channelFactory.CreateChannel();
            (proxy as ICommunicationObject).Open();
            for (int i = 1; i < 6; i++)
            {
                ThreadPool.QueueUserWorkItem(state =>
                {
                    int clientId = Interlocked.Increment(ref clientIdIndex);
                    EventMonitor.Send(clientId, EventType.StartCall);
                    using (OperationContextScope contextScope = new OperationContextScope(proxy as IContextChannel))
                    {
                        MessageHeader<int> messageHeader = new MessageHeader<int>(clientId);
                        OperationContext.Current.OutgoingMessageHeaders.Add(messageHeader.GetUntypedHeader(EventMonitor.CientIdHeaderLocalName, EventMonitor.CientIdHeaderNamespace));
                        proxy.Add(1, 2);
                    }
                    EventMonitor.Send(clientId, EventType.EndCall);
                }, null);
            }
        }

        private void ConcurrentCallViaDiffProxy()
        {
            for (int i = 1; i <= 5; i++)
            {
                ThreadPool.QueueUserWorkItem(state =>
                {
                    int clientId = Interlocked.Increment(ref clientIdIndex);
                    ICalculator proxy = _channelFactory.CreateChannel();
                    using (proxy as IDisposable)
                    {
                        EventMonitor.Send(clientId, EventType.StartCall);
                        using (OperationContextScope contextScope = new OperationContextScope(proxy as IContextChannel))
                        {
                            MessageHeader<int> messageHeader = new MessageHeader<int>(clientId);
                            OperationContext.Current.OutgoingMessageHeaders.Add(messageHeader.GetUntypedHeader(EventMonitor.CientIdHeaderLocalName, EventMonitor.CientIdHeaderNamespace));
                            proxy.Add(1, 2);
                        }
                        EventMonitor.Send(clientId, EventType.EndCall);
                    }
                }, null);
            }
        }

        public void ReceiveMonitoringNotification(object sender, MonitorEventArgs args)
        {
            string message = string.Format("{0, -15}{1, -20}{2}", args.ClientId, args.EventTime.ToLongTimeString(), args.EventType);
            _syncContext.Post(state => this.listBoxExecutionProgress.Items.Add(message), null);
        }
    }
}
